local maximum_speed = 3
        
colors = {
	white = {r = 1, g = 1, b = 1},
	black = {r = 0, g = 0, b = 0},
	darkgrey = {r = 0.25, g = 0.25, b = 0.25},
	grey = {r = 0.5, g = 0.5, b = 0.5},
	lightgrey = {r = 0.75, g = 0.75, b = 0.75},

	red = {r = 1, g = 0, b = 0},
	darkred = {r = 0.5, g = 0, b = 0},
	lightred = {r = 1, g = 0.5, b = 0.5},
	green = {r = 0, g = 1, b = 0},
	darkgreen = {r = 0, g = 0.5, b = 0},
	lightgreen = {r = 0.5, g = 1, b = 0.5},
	blue = {r = 0, g = 0, b = 1},
	darkblue = {r = 0, g = 0, b = 0.5},
	lightblue = {r = 0.5, g = 0.5, b = 1},

	orange = {r = 1, g = 0.55, b = 0.1},
	yellow = {r = 1, g = 1, b = 0},
	pink = {r = 1, g = 0, b = 1},
	purple = {r = 0.6, g = 0.1, b = 0.6},
	brown = {r = 0.6, g = 0.4, b = 0.1},
}


local function init_globals()
	if not global.specified_speed then global.specified_speed = 0.3 end
	if not global.stored_speed then global.stored_speed = 4 end
end


local function init_player(player)
	if player.connected then build_gui(player) end
end


local function init_players()
	for _, player in pairs(game.players) do
		init_player(player)
	end
end


local function on_init() 
	init_globals()
	init_players()
end

script.on_init(on_init)


local function on_configuration_changed(data)
	if data.mod_changes ~= nil then
		local changes = data.mod_changes["vlads_AutoTime"]
		if changes ~= nil then
			init_globals()
			
			init_players()

			update_guis()			
		end
	end
end

script.on_configuration_changed(on_configuration_changed)


local function on_player_created(event)
	init_player(game.players[event.player_index])
end

script.on_event(defines.events.on_player_created, on_player_created)


--prev_positions = { }
local latest_vehicle = { }

local function on_tick(event)
	if event.tick % math.ceil(16*game.speed) == 0 then
		local min_speed = nil
		for _, player in pairs(game.players) do
		    local is_in_vehicle = player.vehicle and (latest_vehicle[player.index])  and (latest_vehicle[player.index].vehicle==player.vehicle)
            local is_in_vehicle_damaged = is_in_vehicle and (latest_vehicle[player.index].health < (player.vehicle.health  + (player.vehicle.grid and player.vehicle.grid.shield or 0)))
			if (is_in_vehicle and (is_in_vehicle_damaged or latest_vehicle[player.index].active_tick > event.tick - 60 * 5))
            or (player.character and (player.character.in_combat 
                or (math.max(player.character.tick_of_last_damage, player.character.tick_of_last_attack) > event.tick - 60 * 5))) then
				min_speed = 1
			end
		  latest_vehicle[player.index]  = { vehicle = player.vehicle, health = player.vehicle and (player.vehicle.health + (player.vehicle.grid and player.vehicle.grid.shield or 0)) or 0, active_tick = 0  }
          if is_in_vehicle_damaged then 
            latest_vehicle[player.index].active_tick = event.tick 
          end
			
			if not min_speed then
				--local entry = prev_positions[player.index]
				--local pos = player.position
				--if entry == nil or entry.x~=pos.x or entry.y~=pos.y then
				if player.walking_state.walking 
						or (player.riding_state 
									and player.riding_state.acceleration~=defines.riding.acceleration.nothing
									--and player.riding_state.acceleration~=defines.riding.acceleration.braking
									) 
						or player.mining_state.mining
						or player.shooting_state.state ~= defines.shooting.not_shooting
						or player.repair_state.repairing
						then 
					--entry = {x=pos.x,y=pos.y}
					--prev_positions[player.index] = entry
					if player.riding_state and player.riding_state.acceleration==defines.riding.acceleration.braking then
						min_speed = 0.6
					else
						min_speed = 1
					end
				end
			end			
			if (not min_speed or min_speed < 0.5) and player.vehicle then
				min_speed = 0.4
			end
		end
		if min_speed and game.speed <= min_speed + 0.01 then
			game.speed = min_speed
			update_guis()			
		else
			if global.specified_speed ~= nil then
				if (game.speed > global.specified_speed) then
					game.speed = math.max(global.specified_speed, game.speed - 0.09)
				else
					game.speed = global.specified_speed
				end
				update_guis()
			end
		end				
	end	
end

script.on_event(defines.events.on_tick, on_tick)

local function toggle_speed()
	if not global.specified_speed then global.specified_speed = game.speed end
	if global.specified_speed >= 1 then game.speed = 0.3 else game.speed = maximum_speed end
	global.specified_speed = game.speed
	update_guis()
end

local function round(num, numDecimalPlaces)
  local mult = 10^(numDecimalPlaces or 0)
  return math.floor(num * mult + 0.5) / mult
end

local function on_gui_click(event)
	if event.element.name == "va_slower" then
		if game.speed > 2 then
			game.speed = round(game.speed - 1)
		elseif game.speed > 1 then
			game.speed = 1
		else
			game.speed = math.max(0.1,round(game.speed - 0.1,1))
		end
		if game.speed ~= 1 then global.stored_speed = game.speed end
		global.specified_speed = game.speed
		update_guis()
		
	elseif event.element.name == "va_faster" then
		if game.speed < 1 then
			if game.speed >= 0.85 then game.speed = 1 else game.speed = round(game.speed + 0.1,1) end
		elseif game.speed < maximum_speed then game.speed = math.min(maximum_speed, round(game.speed+1)) end
		if game.speed ~= 1 then global.stored_speed = game.speed end
		global.specified_speed = game.speed
		update_guis()

	elseif event.element.name == "va_speed" then
		toggle_speed()
	end
	
end

script.on_event(defines.events.on_gui_click, on_gui_click)

function build_gui(player)
	local flowgui = player.gui.top.autotime_flow
	
	if flowgui == nil then
		flowgui = player.gui.top.add({type = "flow", name = "autotime_flow", direction = "horizontal", style = "autotime_flow_style"})
		flowgui.add({type = "button", name = "va_slower", caption = "<" , font_color = colors.white, style = "autotime_button_style"})
		flowgui.add({type = "button", name = "va_faster", caption = ">" , font_color = colors.white, style = "autotime_button_style"})
		flowgui.add({type = "button", name = "va_speed", caption = "x1" , font_color = colors.white, style = "autotime_button_style"}).style.minimal_width = 55
	end
	return flowgui
end
	
function update_guis()
		for _, player in pairs(game.players) do
			if player.connected then
				local flow = build_gui(player)
				local s
				
				if game.speed == 1 then
					flow.va_speed.caption = "x1"
					flow.va_speed.style.font_color = colors.white
				elseif game.speed < 1 then
					s = string.format("x%1.1f", game.speed )
					flow.va_speed.caption = s
					flow.va_speed.style.font_color = colors.green
				elseif game.speed > 1 then
					s = string.format("x%1.0f", game.speed )
					flow.va_speed.caption = s
					flow.va_speed.style.font_color = colors.lightred
				end		
			end
	end
end

script.on_event("auto-time-toggle", function (event)
	toggle_speed()
end)

-------------------------------------------------------------------------------------

